/*
    Video2P -- video hosting engine for I2P network.
    Copyright (C) 2023 - 2025 Video2P Team

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "mysql.hpp"

V2PMySQLInitException::V2PMySQLInitException(const char *error_text) throw()
    : m_text(error_text)
{
}

V2PMySQLInitException::~V2PMySQLInitException() throw()
{
}

const char *
V2PMySQLInitException::what() const throw()
{
	return m_text.get();
}

V2PMySQLConnectionException::V2PMySQLConnectionException(
    const char *error_text) throw()
    : m_text(error_text)
{
}

V2PMySQLConnectionException::~V2PMySQLConnectionException() throw()
{
}

const char *
V2PMySQLConnectionException::what() const throw()
{
	return m_text.get();
}

V2PMySQLResultException::V2PMySQLResultException(const char *error_text) throw()
    : m_text(error_text)
{
}

V2PMySQLResultException::~V2PMySQLResultException() throw()
{
}

const char *
V2PMySQLResultException::what() const throw()
{
	return m_text.get();
}

V2PMySQLQueryException::V2PMySQLQueryException(const char *error_text) throw()
    : m_text(error_text)
{
}

V2PMySQLQueryException::~V2PMySQLQueryException() throw()
{
}

const char *
V2PMySQLQueryException::what() const throw()
{
	return m_text.get();
}

V2PMySQLResultRow::V2PMySQLResultRow(MYSQL_ROW row,
    unsigned num_columns) throw()
    : m_row(row)
    , m_num_columns(num_columns)
{
}

size_t
V2PMySQLResultRow::getSize() const throw()
{
	return m_num_columns;
}

const char *
V2PMySQLResultRow::getColumn(size_t index) const throw()
{
	return m_row[index];
}

V2PMySQLResultSet::V2PMySQLResultSet(MYSQL_RES *result) throw()
    : m_result(result)
    , m_num_rows(mysql_num_rows(m_result))
    , m_cur_row_index(0)
{
}

V2PMySQLResultSet::~V2PMySQLResultSet() throw()
{
	mysql_free_result(m_result);
}

V2PMySQLResultRow *
V2PMySQLResultSet::getNextRow() throw(V2PMySQLResultException)
{
	if (m_cur_row_index++ < m_num_rows) {
		return new V2PMySQLResultRow(mysql_fetch_row(m_result),
		    mysql_num_fields(m_result));
	} else {
		throw V2PMySQLResultException("ResultSet out of range");
	}
}

V2PMySQLResultRow *
V2PMySQLResultSet::getRowByIndex(size_t index) throw(V2PMySQLResultException)
{
	if (index < m_num_rows) {
		return new V2PMySQLResultRow(mysql_fetch_row(m_result),
		    mysql_num_fields(m_result));
	} else {
		throw V2PMySQLResultException("ResultSet out of range");
	}
}

size_t
V2PMySQLResultSet::getSize() const throw()
{
	return m_num_rows;
}

size_t
V2PMySQLResultSet::getCurrentIndex() const throw()
{
	return m_cur_row_index;
}

V2PMySQLConnection::V2PMySQLConnection() throw()
    : m_connection(NULL)
{
}

V2PMySQLConnection::V2PMySQLConnection(const char *host, const char *user,
    const char *passwd, const char *db, unsigned int port,
    const char *unix_socket,
    unsigned long client_flag) throw(V2PMySQLInitException,
    V2PMySQLConnectionException)
    : m_connection(NULL)
    , m_host(host)
    , m_user(user)
    , m_passwd(passwd)
    , m_db(db)
    , m_port(port)
    , m_unix_socket(unix_socket)
    , m_client_flag(client_flag)
{
	reconnect();
}

void
V2PMySQLConnection::connect(const char *host, const char *user,
    const char *passwd, const char *db, unsigned int port,
    const char *unix_socket,
    unsigned long client_flag) throw(V2PMySQLInitException,
    V2PMySQLConnectionException)

{
	m_host = V2PString(host);
	m_user = V2PString(user);
	m_passwd = V2PString(passwd);
	m_db = V2PString(db);
	m_port = port;
	m_unix_socket = V2PString(unix_socket);
	m_client_flag = client_flag;
	reconnect();
}

void
V2PMySQLConnection::reconnect() throw(V2PMySQLInitException,
    V2PMySQLConnectionException)
{
	mysql_close(m_connection);
	if (!(m_connection = mysql_init(NULL))) {
		throw V2PMySQLInitException(mysql_error(m_connection));
	}
	MYSQL *conn_res = mysql_real_connect(m_connection, m_host.get(),
	    m_user.get(), m_passwd.get(), m_db.get(), m_port,
	    m_unix_socket.get(), m_client_flag);
	if (conn_res == NULL) {
		V2PMySQLConnectionException ex(mysql_error(m_connection));
		mysql_close(m_connection);
		throw ex;
	}
}

int
V2PMySQLConnection::ping() const throw()
{
	return mysql_ping(m_connection);
}

V2PMySQLResultSet *
V2PMySQLConnection::query(
    const char *sql) throw(V2PMySQLQueryException, V2PMySQLResultException)
{
	if (mysql_query(m_connection, sql)) {
		throw V2PMySQLQueryException(mysql_error(m_connection));
	}

	MYSQL_RES *result = mysql_store_result(m_connection);
	if (mysql_errno(m_connection)) {
		throw V2PMySQLResultException(mysql_error(m_connection));
	}

	return new V2PMySQLResultSet(result);
}
