/*
    Video2P -- video hosting engine for I2P network.
    Copyright (C) 2023 - 2025 Video2P Team

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef VIDEO2P_MYSQL_HPP
#define VIDEO2P_MYSQL_HPP

#include <mysql/mysql.h>

#include "exception.hpp"
#include "string.hpp"

class V2PMySQLException : public V2PException {
public:
	virtual ~V2PMySQLException() throw() {};
	virtual const char *what() const throw() = 0;
};

class V2PMySQLInitException : public V2PMySQLException {
private:
	V2PString m_text;

public:
	V2PMySQLInitException(const char *error_text) throw();
	~V2PMySQLInitException() throw();
	const char *what() const throw();
};

class V2PMySQLConnectionException : public V2PMySQLException {
private:
	V2PString m_text;

public:
	V2PMySQLConnectionException(const char *error_text) throw();
	~V2PMySQLConnectionException() throw();
	const char *what() const throw();
};

class V2PMySQLResultException : public V2PMySQLException {
private:
	V2PString m_text;

public:
	V2PMySQLResultException(const char *error_text) throw();
	~V2PMySQLResultException() throw();
	const char *what() const throw();
};

class V2PMySQLQueryException : public V2PMySQLException {
private:
	V2PString m_text;

public:
	V2PMySQLQueryException(const char *error_text) throw();
	~V2PMySQLQueryException() throw();
	const char *what() const throw();
};

// Valid only util valid V2PMySQLResultSet
class V2PMySQLResultRow {
public:
	typedef unsigned index_t;

private:
	MYSQL_ROW m_row;
	index_t m_num_columns;

public:
	V2PMySQLResultRow(MYSQL_ROW row, unsigned num_columns) throw();

private:
	V2PMySQLResultRow(const V2PMySQLResultRow &);
	V2PMySQLResultRow &operator=(const V2PMySQLResultRow &);

public:
	size_t getSize() const throw();
	const char *getColumn(size_t index) const throw();
};

class V2PMySQLResultSet {
public:
	typedef uint64_t index_t;

private:
	MYSQL_RES *m_result;
	index_t m_num_rows;
	index_t m_cur_row_index;

public:
	V2PMySQLResultSet(MYSQL_RES *result) throw();

private:
	V2PMySQLResultSet(const V2PMySQLResultSet &);
	V2PMySQLResultSet &operator=(const V2PMySQLResultSet &);

public:
	~V2PMySQLResultSet() throw();
	V2PMySQLResultRow *getNextRow() throw(V2PMySQLResultException);
	V2PMySQLResultRow *getRowByIndex(size_t index) throw(
	    V2PMySQLResultException);
	size_t getSize() const throw();
	size_t getCurrentIndex() const throw();
};

class V2PMySQLConnection {
private:
	MYSQL *m_connection;
	V2PString m_host;
	V2PString m_user;
	V2PString m_passwd;
	V2PString m_db;
	unsigned int m_port;
	V2PString m_unix_socket;
	unsigned long m_client_flag;

public:
	V2PMySQLConnection() throw();

	V2PMySQLConnection(const char *host, const char *user,
	    const char *passwd, const char *db, unsigned int port,
	    const char *unix_socket,
	    unsigned long client_flag) throw(V2PMySQLInitException,
	    V2PMySQLConnectionException);

private:
	V2PMySQLConnection(const V2PMySQLConnection &);
	V2PMySQLConnection &operator=(const V2PMySQLConnection &);

public:
	void connect(const char *host, const char *user, const char *passwd,
	    const char *db, unsigned int port, const char *unix_socket,
	    unsigned long client_flag) throw(V2PMySQLInitException,
	    V2PMySQLConnectionException);
	void reconnect() throw(V2PMySQLInitException,
	    V2PMySQLConnectionException);
	int ping() const throw();
	V2PMySQLResultSet *query(const char *sql) throw(V2PMySQLQueryException,
	    V2PMySQLResultException);
};

#endif // VIDEO2P_MYSQL_HPP
