/*
    Video2P -- video hosting engine for I2P network.
    Copyright (C) 2023 - 2025 Video2P Team

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef VIDEO2P_VECTOR_HPP
#define VIDEO2P_VECTOR_HPP

#include <stddef.h>

#include "memory.hpp"

template <class T> class V2PVector {
public:
	typedef size_t index_t;
	static const index_t before_0_index;

private:
	T *m_array;
	index_t m_size;

public:
	V2PVector() throw()
	    : m_array(NULL)
	    , m_size(0)
	{
	}

	V2PVector(index_t size)
	    : m_array(m_alloc(sizeof(T) * size))
	    , m_size(size)
	{
		index_t i;
		try {
			for (i = 0; i < m_size; ++i) {
				new (m_array + i) T();
			}
		} catch (...) {
			m_destroy_since_index(i - 1);
			V2PAllocator::free(m_array);
			throw;
		}
	}

	V2PVector(const V2PVector &other)
	    : m_array(m_alloc(sizeof(T) * other.m_size))
	    , m_size(other.m_size)
	{
		index_t i;
		try {
			for (i = 0; i < m_size; ++i) {
				new (m_array + i) T(other.m_array[i]);
			}
		} catch (...) {
			m_destroy_since_index(i - 1);
			V2PAllocator::free(m_array);
			throw;
		}
	}

	V2PVector &operator=(const V2PVector &other)
	{
		V2PAllocator::free(m_array);
		m_array = m_alloc(sizeof(T) * other.m_size);
		m_size = other.m_size;
		index_t i;
		try {
			for (i = 0; i < m_size; ++i) {
				new (m_array + i) T(other.m_array[i]);
			}
		} catch (...) {
			m_destroy_since_index(i - 1);
			V2PAllocator::free(m_array);
			throw;
		}
		return *this;
	}

	~V2PVector() throw()
	{
		m_destroy_since_index(m_size - 1);
		V2PAllocator::free(m_array);
	}

	index_t getSize() const throw() { return m_size; }

	T &operator[](index_t index) const throw() { return m_array[index]; }

private:
	T *m_alloc(size_t size) throw(V2PAllocatorException)
	{
		return reinterpret_cast<T *>(V2PAllocator::malloc(size));
	}

	void m_destroy_since_index(index_t index) throw()
	{
		for (index_t i = index; i != before_0_index; --i) {
			m_array[i].~T();
		}
	}
};

template <class T>
const typename V2PVector<T>::index_t V2PVector<T>::before_0_index = -1;

#endif // VIDEO2P_VECTOR_HPP
